import json
import boto3
from botocore.exceptions import ClientError

from pprint import pprint
import logging
import os

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

TOPIC_ARN = os.environ['TOPIC_ARN']
TABLE_NAME = os.environ['TABLE_NAME']

def get_notify_preference(user):
    # Query the DynamoDB table and check if the user has a defined subscription preference
    dynamodb = boto3.resource('dynamodb')

    table = dynamodb.Table(TABLE_NAME)

    try:
        # the user field should match the TrasferFamily server username
        response = table.get_item(Key={'user': user})
        if 'Item' in response:
            item = response['Item']
            return item
        
        return []
    except ClientError as err:
        logger.warn(err.response['Error']['Message'])
        return []

def subscribe(sns_client, username, topic, protocol, endpoint):
    # subscribe the user to the SNS topic if thhe user has not been subscribed before
    try:
        response = sns_client.list_subscriptions_by_topic(TopicArn=topic)
        subscriptions = response["Subscriptions"]

        # check if the user is subscribed
        sub_found=False
        for s_endpoint in subscriptions:
            if s_endpoint['Endpoint'] == endpoint: 
                sub_found=True       
                break
        # if not subscribed, add new subscription
        if not sub_found:
            subscription = sns_client.subscribe(
                TopicArn=topic,
                Protocol=protocol,
                Endpoint=endpoint,
                ReturnSubscriptionArn=True)['SubscriptionArn']
            # the filter is necessary to notify user using the prefered notification protocol only
            if subscription:
                sns_client.set_subscription_attributes(
                    SubscriptionArn = subscription, 
                    AttributeName = 'FilterPolicy', 
                    AttributeValue = json.dumps({"user": [username]})
                )
    except ClientError as err:
        logger.warn(err.response['Error']['Message'])
        return ("FAILURE")
    else:
        return ("SUCCESS")

def notify_user(user_pref,fileLocationKey):
    try:
        # Notifications can be delivered with https, email and sms protocols
        if user_pref['protocol']=='https' or user_pref['protocol']=='email' or user_pref['protocol']=='sms':
            sns_client = boto3.client('sns')
            topic_arn = TOPIC_ARN
            response = subscribe(sns_client, user_pref['user'],topic_arn, user_pref['protocol'], user_pref['endpoint'])
            if response=="SUCCESS":
                # buld JSON notification to be used for https protocol
                data = "File:{filekey} was uploaded successfully".format(filekey=fileLocationKey)
                notification = {"user": user_pref['user'], "message": data}
                # publish the message to the SNS topic
                # the attribute of the message wil be used by the filter added to the subscription 
                response = sns_client.publish (
                    TargetArn = topic_arn,
                    Subject = "AWS Transfer Family - File Upload Successful",
                    Message = json.dumps({
                        'default': "File:{filekey} was uploaded successfully by {user}".format(filekey=fileLocationKey, user=user_pref['user']),
                        'https':json.dumps(notification),
                        'email': "File:{filekey} was uploaded successfully by {user}".format(filekey=fileLocationKey, user=user_pref['user']),
                        'sms': "File:{filekey} uploaded by {user}".format(filekey=fileLocationKey, user=user_pref['user']) # shorter message suitable for sms
                    }),
                    MessageAttributes={
                            'user': {
                                'DataType': 'String',
                                'StringValue': user_pref['user']
                            }
                        },
                    MessageStructure = 'json'
                )
            else:
                return ("FAILURE")
    except Exception as e:
        return ("FAILURE")
    else:
        return("SUCCESS")

def lambda_handler(event, context):
    userName=event['serviceMetadata']['transferDetails']['userName']
    fileLocationKey=event['fileLocation']['key']
    response = "FAILURE"
    
    # get user notification preference
    user_pref = get_notify_preference(userName)
    if user_pref:
        # start notifying user and get back the result as response
        response = notify_user(user_pref,fileLocationKey)
    
    # return the result/state to Trasnfer Family managed workflow
    transfer = boto3.client('transfer')
    response = transfer.send_workflow_step_state(
            WorkflowId=event['serviceMetadata']['executionDetails']['workflowId'],
            ExecutionId=event['serviceMetadata']['executionDetails']['executionId'],
            Token=event['token'],
            Status=response
        )    
        
    return {
      'statusCode': 200,
      'body': json.dumps(response)
    }